<?php

/*
 * ##################################################
 * #                                                #
 * # rkCSD-WebsiteEngine_2                          #
 * # Programmed for the new web decade i.o.b. rkCSD #
 * #                                                #
 * # Copyright (C) by D. Morabito & R. Knipschild   #
 * #                                                #
 * # staff@newsocialife.com         email@rkcsd.com #
 * # www.newsocialfie.com   www.ihr-edv-fachmann.de #
 * #                                                #
 * ##################################################
 * 
 * File: rkCSD.php
 * Version: 1.0.0
 * Last modified: 2013/03/12 12:20 GMT
 * Author: Danny Morabito, René Knipschild
 * 
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

class rkCSD {
	public $config;
	public $routes;
	public $router;
	public $version = "2.0.0";
	function __construct() {
		$this->config = parse_ini_file("config.ini", true);
		$this->routes = array();
	}
	function version() {
		return $this->version;
	}
	function is_up_to_date() {
		// TODO: check if the library is up to date with the version in the git repository and if not update it
	}
	function add($protocol, $extras = array()) {
		$protocol = strtolower($protocol);
		if(isset($this->config[strtolower($protocol)]))
			$extras = $this->config[strtolower($protocol)];
		if(!isset($this->$protocol)) {
			require $this->getconfig("engine", "extensions").$protocol.".php";
			if($protocol == "cms")
				return $this->cms();
			if($protocol == "login")
				$this->$protocol = new $protocol($this, $extras);
			else
				$this->$protocol = new $protocol($extras);
			if($protocol == "login")
				$this->routes = array_merge($this->routes, $this->login->addRoutesToMain());
			if($protocol == "mysql")
				$this->db = $this->mysql;
		}else
			return false;
	}
	function getconfigarray($element) {
		return $this->config[$element];
	}
	function getconfig($group, $element) {
		return $this->config[$group][$element];
	}
	function rkError($msg, $type = E_USER_WARNING) {
		trigger_error($msg, $type);
		die;
	}
	function config($section) {
		return $this->config[$section];
	}
	function classorerror($class) {
		$class = strtolower($class);
		if(!isset($this->$class))
			$this->rkError("The protocol {$class} isn't included", E_USER_WARNING);
	}
	function get($path, $function) {
		$this->classorerror("router");
		$router = $this->router;
		$this->routes[$path] = array("GET", $function);
		return $this->routes;
	}
	function post($path, $function) {
		$this->classorerror("router");
		$router = $this->router;
		$this->routes[$path] = array("POST", $function);
		return $this->routes;
	}
	function redirect($path) {
		$this->classorerror("router");
		$router = $this->router;
		return $router->redirect($path);
	}
	function debug($onoff) {
		if($onoff)  {
			error_reporting(E_ALL);
			ini_set("display_errors", 1);
		}else{
			error_reporting(0);
			ini_set("display_errors", 0);
		}
	}
	function edit($file) {
		return file_get_contents($file);
	}
	function render($template, $args = array()) {
		$this->classorerror("templating");
		$templating = $this->templating;
		$content = $templating->render("pagedesign.html", array_merge($this->config("site"), array_merge($args, array("page" => $template))));
		/* $content = $templating->render("header.html", $this->config("site"));
		$content .= $templating->render($template, $args);
		$content .= $templating->render("footer.html", $this->config("site")); */
		return $content;
	}
	function getAllTemplates() {
		$templates = glob($this->config["templating"]["templates"]."*.html");
		foreach($templates as $k => $v) {
			$templates[$k] = str_replace($this->config["templating"]["templates"], "", $templates[$k]);
			if($templates[$k] == "pagedesign.html" || $templates[$k] == "tinymce.html" || $templates[$k] == "login.html" || $templates[$k] == "editfile_choose.html") {
					unset($templates[$k]);
					continue;
			}
			$templates[$k] = str_replace(".html", "", $templates[$k]);
		}
		return $templates;
	}
	function sql() {
		return $this->mysql;
	}
	function mysql() {
		return $this->mysql;
	}
	function run() {
		if(!empty($this->routes)) {
			foreach($this->routes as $k => $v) {
				if(strtoupper($v[0]) == "GET")
					$this->router->get($k, $v[1]);
				if(strtoupper($v[0]) == "POST")
					$this->router->post($k, $v[1]);
			}
		}
		if(isset($this->router))
			$this->router->run();
	}
	function login($user, $password) {
		$this->classorerror("mysql");
		$this->classorerror("login");
		$this->classorerror("session");
		return $this->login->logindo($user, $this->login->password($password));
	}
	function cms() { // this will start the cms
		$this->classorerror("router");
		$this->classorerror("templating");
		$this->classorerror("login");
		$lib = $this;
		if($this->login->logged_in()) {
			$this->routes["/edit/{file}".$this->config["templating"]["extension"]] = array("GET", function($file) use($lib) {
					return $lib->render(
						"tinymce.html", 
						array(
							"content" => $lib->edit($lib->config["templating"]["templates"].$file.$lib->config["templating"]["extension"]),
							"rkCMSFile" => $file
						)
					);
			});
			$this->routes["/edit/"] = array("GET", function() use($lib) {
				return $lib->render("editfile_choose.html", array("files" => $lib->getAllTemplates()));
			});
			$this->routes["/edit/save"] = array("POST", function() use($lib) {
				$_POST["rkCMSFile"] = preg_replace("/[^A-Za-z0-9]/", "", $_POST["rkCMSFile"]);
				$_POST["rkCMSFile"] = $lib->config["templating"]["templates"].$_POST["rkCMSFile"].".html";
				$content = $_POST["page"];
				file_put_contents($_POST["rkCMSFile"], $content);
				return $lib->redirect("/");
			});
		} else {
			$this->routes["/edit"] = array("GET", 
				function() use($lib) {
					return $lib->render(
						"login.html", 
						$lib->login->getLoginArray()
					);
				}
			);
		}
	}
}

?>